@@ -398,7 +398,7 @@ impl crate::RenderInner<Context> for RenderBundleEncoder {
 impl crate::RenderPassInner<Context> for RenderPass {
     fn set_blend_constant(&mut self, color: wgt::Color) {
         self.0
-            .set_blend_color_with_gpu_color_dict(&map_color(color));
+            .set_blend_constant_with_gpu_color_dict(&map_color(color));
     }
     fn set_scissor_rect(&mut self, x: u32, y: u32, width: u32, height: u32) {
         self.0.set_scissor_rect(x, y, width, height);
@@ -676,17 +676,17 @@ fn map_blend_factor(factor: wgt::BlendFactor) -> web_sys::GpuBlendFactor {
     match factor {
         BlendFactor::Zero => bf::Zero,
         BlendFactor::One => bf::One,
-        BlendFactor::Src => bf::SrcColor,
-        BlendFactor::OneMinusSrc => bf::OneMinusSrcColor,
+        BlendFactor::Src => bf::Src,
+        BlendFactor::OneMinusSrc => bf::OneMinusSrc,
         BlendFactor::SrcAlpha => bf::SrcAlpha,
         BlendFactor::OneMinusSrcAlpha => bf::OneMinusSrcAlpha,
-        BlendFactor::Dst => bf::DstColor,
-        BlendFactor::OneMinusDst => bf::OneMinusDstColor,
+        BlendFactor::Dst => bf::Dst,
+        BlendFactor::OneMinusDst => bf::OneMinusDst,
         BlendFactor::DstAlpha => bf::DstAlpha,
         BlendFactor::OneMinusDstAlpha => bf::OneMinusDstAlpha,
         BlendFactor::SrcAlphaSaturated => bf::SrcAlphaSaturated,
-        BlendFactor::Constant => bf::BlendColor,
-        BlendFactor::OneMinusConstant => bf::OneMinusBlendColor,
+        BlendFactor::Constant => bf::Constant,
+        BlendFactor::OneMinusConstant => bf::OneMinusConstant,
     }
 }
 
@@ -754,8 +754,8 @@ fn map_vertex_format(format: wgt::VertexFormat) -> web_sys::GpuVertexFormat {
     }
 }
 
-fn map_input_step_mode(mode: wgt::VertexStepMode) -> web_sys::GpuInputStepMode {
-    use web_sys::GpuInputStepMode as sm;
+fn map_vertex_step_mode(mode: wgt::VertexStepMode) -> web_sys::GpuVertexStepMode {
+    use web_sys::GpuVertexStepMode as sm;
     use wgt::VertexStepMode;
     match mode {
         VertexStepMode::Vertex => sm::Vertex,
@@ -851,7 +851,7 @@ fn map_store_op(store: bool) -> web_sys::GpuStoreOp {
     if store {
         web_sys::GpuStoreOp::Store
     } else {
-        web_sys::GpuStoreOp::Clear
+        web_sys::GpuStoreOp::Discard
     }
 }
 
@@ -865,8 +865,8 @@ fn map_map_mode(mode: crate::MapMode) -> u32 {
 type JsFutureResult = Result<wasm_bindgen::JsValue, wasm_bindgen::JsValue>;
 
 fn future_request_adapter(result: JsFutureResult) -> Option<Sendable<web_sys::GpuAdapter>> {
-    match result {
-        Ok(js_value) => Some(Sendable(web_sys::GpuAdapter::from(js_value))),
+    match result.and_then(wasm_bindgen::JsCast::dyn_into) {
+        Ok(adapter) => Some(Sendable(adapter)),
         Err(_) => None,
     }
 }
@@ -887,6 +887,30 @@ fn future_map_async(result: JsFutureResult) -> Result<(), crate::BufferAsyncErro
     result.map(|_| ()).map_err(|_| crate::BufferAsyncError)
 }
 
+impl Context {
+    pub fn instance_create_surface_from_canvas(
+        &self,
+        canvas: &web_sys::HtmlCanvasElement,
+    ) -> <Self as crate::Context>::SurfaceId {
+        let context: wasm_bindgen::JsValue = match canvas.get_context("webgpu") {
+            Ok(Some(ctx)) => ctx.into(),
+            _ => panic!("expected to get context from canvas"),
+        };
+        Sendable(context.into())
+    }
+
+    pub fn instance_create_surface_from_offscreen_canvas(
+        &self,
+        canvas: &web_sys::OffscreenCanvas,
+    ) -> <Self as crate::Context>::SurfaceId {
+        let context: wasm_bindgen::JsValue = match canvas.get_context("webgpu") {
+            Ok(Some(ctx)) => ctx.into(),
+            _ => panic!("expected to get context from canvas"),
+        };
+        Sendable(context.into())
+    }
+}
+
 impl crate::Context for Context {
     type AdapterId = Sendable<web_sys::GpuAdapter>;
     type DeviceId = Sendable<web_sys::GpuDevice>;
@@ -949,11 +973,7 @@ impl crate::Context for Context {
             .expect("expected to find single canvas")
             .into();
         let canvas_element: web_sys::HtmlCanvasElement = canvas_node.into();
-        let context: wasm_bindgen::JsValue = match canvas_element.get_context("gpupresent") {
-            Ok(Some(ctx)) => ctx.into(),
-            _ => panic!("expected to get context from canvas"),
-        };
-        Sendable(context.into())
+        self.instance_create_surface_from_canvas(&canvas_element)
     }
 
     fn adapter_is_surface_supported(
@@ -1019,7 +1039,7 @@ impl crate::Context for Context {
             ),
             (wgt::Features::TIMESTAMP_QUERY, Gfn::TimestampQuery),
         ];
-        let non_guaranteed_features = possible_features
+        let required_features = possible_features
             .iter()
             .copied()
             .flat_map(|(flag, value)| {
@@ -1030,7 +1050,7 @@ impl crate::Context for Context {
                 }
             })
             .collect::<js_sys::Array>();
-        mapped_desc.non_guaranteed_features(&non_guaranteed_features);
+        mapped_desc.required_features(&required_features);
 
         if let Some(label) = desc.label {
             mapped_desc.label(label);
@@ -1067,8 +1087,8 @@ impl crate::Context for Context {
             max_storage_buffers_per_shader_stage: limits.max_storage_buffers_per_shader_stage(),
             max_storage_textures_per_shader_stage: limits.max_storage_textures_per_shader_stage(),
             max_uniform_buffers_per_shader_stage: limits.max_uniform_buffers_per_shader_stage(),
-            max_uniform_buffer_binding_size: limits.max_uniform_buffer_binding_size(),
-            max_storage_buffer_binding_size: limits.max_storage_buffer_binding_size(),
+            max_uniform_buffer_binding_size: limits.max_uniform_buffer_binding_size() as u32,
+            max_storage_buffer_binding_size: limits.max_storage_buffer_binding_size() as u32,
             max_vertex_buffers: limits.max_vertex_buffers(),
             max_vertex_attributes: limits.max_vertex_attributes(),
             max_vertex_buffer_array_stride: limits.max_vertex_buffer_array_stride(),
@@ -1108,8 +1128,7 @@ impl crate::Context for Context {
         surface: &Self::SurfaceId,
         adapter: &Self::AdapterId,
     ) -> Option<wgt::TextureFormat> {
-        let format =
-            map_texture_format_from_web_sys(surface.0.get_swap_chain_preferred_format(&adapter.0));
+        let format = map_texture_format_from_web_sys(surface.0.get_preferred_format(&adapter.0));
         Some(format)
     }
 
@@ -1120,21 +1139,21 @@ impl crate::Context for Context {
         config: &wgt::SurfaceConfiguration,
     ) {
         let mut mapped =
-            web_sys::GpuSwapChainDescriptor::new(&device.0, map_texture_format(config.format));
+            web_sys::GpuCanvasConfiguration::new(&device.0, map_texture_format(config.format));
         mapped.usage(config.usage.bits());
-        surface.0.configure_swap_chain(&mapped);
+        surface.0.configure(&mapped);
     }
 
     fn surface_get_current_texture(
         &self,
-        _surface: &Self::SurfaceId,
+        surface: &Self::SurfaceId,
     ) -> (
         Option<Self::TextureId>,
         wgt::SurfaceStatus,
         Self::SurfaceOutputDetail,
     ) {
         (
-            None, //TODO: surface.0.get_current_texture(),
+            Some(Sendable(surface.0.get_current_texture())),
             wgt::SurfaceStatus::Good,
             (),
         )
@@ -1144,6 +1163,14 @@ impl crate::Context for Context {
         // Swapchain is presented automatically
     }
 
+    fn surface_texture_discard(
+        &self,
+        _texture: &Self::TextureId,
+        _detail: &Self::SurfaceOutputDetail,
+    ) {
+        // Can't really discard this on the Web
+    }
+
     fn device_features(&self, _device: &Self::DeviceId) -> wgt::Features {
         // TODO
         wgt::Features::empty()
@@ -1163,15 +1190,31 @@ impl crate::Context for Context {
         &self,
         device: &Self::DeviceId,
         desc: &crate::ShaderModuleDescriptor,
+        _shader_bound_checks: wgt::ShaderBoundChecks,
     ) -> Self::ShaderModuleId {
         let mut descriptor = match desc.source {
-            #[cfg(feature = "spirv")]
+            #[cfg(feature = "spirv-web")]
             crate::ShaderSource::SpirV(ref spv) => {
-                web_sys::GpuShaderModuleDescriptor::new(&js_sys::Uint32Array::from(&**spv))
-            }
-            crate::ShaderSource::Wgsl(ref code) => {
-                web_sys::GpuShaderModuleDescriptor::new(&js_sys::JsString::from(&**code))
+                use naga::{back, front, valid};
+
+                let options = naga::front::spv::Options {
+                    adjust_coordinate_space: false,
+                    strict_capabilities: true,
+                    block_ctx_dump_prefix: None,
+                };
+                let spv_parser = front::spv::Parser::new(spv.iter().cloned(), &options);
+                let spv_module = spv_parser.parse().unwrap();
+
+                let mut validator = valid::Validator::new(
+                    valid::ValidationFlags::all(),
+                    valid::Capabilities::all(),
+                );
+                let spv_module_info = validator.validate(&spv_module).unwrap();
+
+                let wgsl_text = back::wgsl::write_string(&spv_module, &spv_module_info).unwrap();
+                web_sys::GpuShaderModuleDescriptor::new(wgsl_text.as_str())
             }
+            crate::ShaderSource::Wgsl(ref code) => web_sys::GpuShaderModuleDescriptor::new(code),
         };
         if let Some(label) = desc.label {
             descriptor.label(label);
@@ -1255,9 +1298,9 @@ impl crate::Context for Context {
                             }
                         };
                         let mut storage_texture = web_sys::GpuStorageTextureBindingLayout::new(
-                            mapped_access,
                             map_texture_format(format),
                         );
+                        storage_texture.access(mapped_access);
                         storage_texture.view_dimension(map_texture_view_dimension(view_dimension));
                         mapped_entry.storage_texture(&storage_texture);
                     }
@@ -1375,7 +1418,7 @@ impl crate::Context for Context {
                     vbuf.array_stride as f64,
                     &mapped_attributes,
                 );
-                mapped_vbuf.step_mode(map_input_step_mode(vbuf.step_mode));
+                mapped_vbuf.step_mode(map_vertex_step_mode(vbuf.step_mode));
                 mapped_vbuf
             })
             .collect::<js_sys::Array>();
@@ -1811,8 +1854,11 @@ impl crate::Context for Context {
                     crate::LoadOp::Load => wasm_bindgen::JsValue::from(web_sys::GpuLoadOp::Load),
                 };
 
-                let mut mapped_color_attachment =
-                    web_sys::GpuRenderPassColorAttachment::new(&load_value, &ca.view.id.0);
+                let mut mapped_color_attachment = web_sys::GpuRenderPassColorAttachment::new(
+                    &load_value,
+                    map_store_op(ca.ops.store),
+                    &ca.view.id.0,
+                );
 
                 if let Some(rt) = ca.resolve_target {
                     mapped_color_attachment.resolve_target(&rt.id.0);
@@ -1894,7 +1940,7 @@ impl crate::Context for Context {
         })
     }
 
-    fn command_encoder_clear_image(
+    fn command_encoder_clear_texture(
         &self,
         _encoder: &Self::CommandEncoderId,
         _texture: &crate::Texture,
